package pl.tph.javatacka.game;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;

//import javax.swing.AbstractAction;
//import javax.swing.Action;
//import javax.swing.ActionMap;
import javax.swing.Box;
//import javax.swing.InputMap;
import javax.swing.JButton;
import javax.swing.JColorChooser;
//import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.JSpinner;
import javax.swing.JTextField;
//import javax.swing.KeyStroke;
import javax.swing.SpinnerNumberModel;

/**
 * Klasa <code>GameMenu</code> tworzy menu, w ktrym gracze mog ustawi swoje 
 * preferencje dotyczce przebiegu gry. Wrd dostpnych opcji s nastpujce:
 * <ul>
 * <li>ilo graczy</li>
 * <li>nazwy poszczeglnych graczy</li>
 * <li>sterowanie (z ograniczonego zbioru par klawiszy)</li>
 * <li>kolory krzywych</li>
 * <li>prdko gry</li>
 * <li>limit punktw</li>
 * </ul>
 * Ta klasa jest rozszerzeniem klasy <code>JFrame</code> z pakietu
 * <em>Swing</em>.
 * <br><br>
 * Podobnie jak wszystkie operacje wykonywane na okienkach <em>Swinga</em>
 * obiekty <code>GameMenu</code> <u>naley tworzy w wtku dystrybucji
 * zdarze</u>. Nowo utworzone menu naley nastpnie uwidoczni posugujc si
 * metod klasy <code>JFrame</code> o nazwie <code>setVisible()</code>.
 * <br><br>
 * Po klikniciu przycisku <em>Start!</em> wszelkie ustawione opcje s
 * wczytywane do obiektu klasy <code>GamePreferences</code>, ktrego referencj
 * mona uzyska przy pomocy metody <code>getGamePreferences()</code>.
 *  
 * @author Tomasz Pieczerak (tph)
 * @see javax.swing.JFrame
 * @see pl.tph.javatacka.game.GamePreferences
 */
public class GameMenu extends JFrame {

	/**
	 * Bezargumentowy konstruktor klasy <code>GameMenu</code>, ktry konfiguruje
	 * okno menu gry.
	 */
	public GameMenu ( ) {
		super("Javatacka");
		done = false;
		pref = null;

		// utwrz nowy panel
		panel = new JPanel();
		panel.setLayout(new FlowLayout(FlowLayout.CENTER, 10, 10));
		panel.setPreferredSize(new Dimension(445,475));
		getContentPane().add(panel);

		// kolejno pola do wpisania nazwy gracza, wybrania koloru jego krzywej
		// oraz przyciski informujce o sterowania gracza
		players = new JTextField[MAX_PLAYERS];
		colors = new JButton[MAX_PLAYERS];
		keys = new JButton[2][MAX_PLAYERS];

		// obiekt wywietlajcy menu wyboru koloru po klikniciu w przycisk
		ActionListener colorActionListener = new ActionListener()
			{
				@Override
				public void actionPerformed ( ActionEvent e ) {
					Object source = e.getSource();
	
					for ( int i = 0; i < MAX_PLAYERS; ++i ) {
						if ( source == colors[i] ) {
							Color color = JColorChooser.showDialog(null,
								"Wybierz kolor", colors[i].getBackground());

							if ( color != null )
								colors[i].setBackground(color);

							break;
						}
					}
				}
			};

		JLabel label;
		label = new JLabel("Nacinij odpowiedni przycisk," +
						   " aby aktywowa gracza :");
		label.setPreferredSize(new Dimension(420, 30));
		panel.add(label);

		// zdefiniuj dla szeciu graczy odpowiednie pola wyboru 
		for ( int i = 0; i < MAX_PLAYERS; ++i ) {
			players[i] = new JTextField("Player"+(i+1), 15);
			colors[i]= new JButton("");

			switch ( i ) {
			case 0:
				keys[0][i]= new JButton("1");
				keys[1][i]= new JButton("Q");
				break;
			case 1:
				keys[0][i]= new JButton("LCtrl");
				keys[1][i]= new JButton("LWin");
				break;
			case 2:
				keys[0][i]= new JButton("C");
				keys[1][i]= new JButton("V");
				break;
			case 3:
				keys[0][i]= new JButton("M");
				keys[1][i]= new JButton(",");
				break;
			case 4:
				keys[0][i]= new JButton("Left");
				keys[1][i]= new JButton("Down");
				break;
			case 5:
				keys[0][i]= new JButton("LPM");
				keys[1][i]= new JButton("PPM");
				break;
			default:
				keys[0][i]= new JButton("???");
				keys[1][i]= new JButton("???");
			}

			colors[i].setPreferredSize(new Dimension(25,25));
			keys[0][i].setPreferredSize(new Dimension(100,25));
			keys[1][i].setPreferredSize(new Dimension(100,25));

			colors[i].addActionListener(colorActionListener);
			keys[0][i].addActionListener(new ActionListener()
				{
					@Override
					public void actionPerformed ( ActionEvent e ) {
						Object source = e.getSource();

						for ( int i = 0; i < MAX_PLAYERS; ++i ) {
							if( source == keys[0][i] ) {
									if (players[i].isEnabled()) {
										players[i].setEnabled(false);
										colors[i].setEnabled(false);
										keys[1][i].setEnabled(false);
									} else {
										players[i].setEnabled(true);
										colors[i].setEnabled(true);
										keys[1][i].setEnabled(true);
									}
							}
						}
					}
				});

			Color color;
			switch ( i ) {
			case 0:
				color = Color.CYAN;
				break;
			case 1:
				color = Color.YELLOW;
				break;
			case 2:
				color = Color.GREEN;
				break;
			case 3:
				color = Color.RED;
				break;
			case 4:
				color = Color.MAGENTA;
				break;
			case 5:
				color = Color.BLUE;
				break;
			default:
				color = Color.GRAY;
			}
			colors[i].setBackground(color);

			players[i].setEnabled(false);
			colors[i].setEnabled(false);
			keys[1][i].setEnabled(false);

			panel.add(players[i]);
			panel.add(colors[i]);
			panel.add(keys[0][i]);
			panel.add(keys[1][i]);
		}

		label = new JLabel("Ustal szybko :");
		label.setPreferredSize(new Dimension(110, 20));
		panel.add(Box.createHorizontalStrut(130));
		panel.add(label);
		panel.add(Box.createHorizontalStrut(130));

		// suwak wyboru prdkoci gry
		speed = new JSlider(20, 100, 30);
		speed.setPreferredSize(new Dimension(425,50));
		speed.setPaintTicks(true);
		speed.setPaintLabels(true);
		speed.setMajorTickSpacing(10);
		speed.setMinorTickSpacing(1);
		speed.setSnapToTicks(true);
		panel.add(speed);

		label = new JLabel("Ustaw limit punktw :");
		label.setPreferredSize(new Dimension(140, 20));
		panel.add(Box.createHorizontalStrut(100));
		panel.add(label);
		panel.add(Box.createHorizontalStrut(100));

		// pole wyboru limitu punktw
		maxScore = new JSpinner(new SpinnerNumberModel(40, 10, 999, 1));
		maxScore.setPreferredSize(new Dimension(50, 25));
		panel.add(Box.createHorizontalStrut(150));
		panel.add(maxScore);
		panel.add(Box.createHorizontalStrut(150));

		// przycisk "Start!"
		panel.add(Box.createRigidArea(new Dimension(400,5)));
		start = new JButton("Start!");
		start.addActionListener(new ActionListener()
			{
			@Override
			public void actionPerformed ( ActionEvent e ) {
				int playersAmount = 0;
				
				try {
					for ( int i = 0; i < MAX_PLAYERS; ++i ) {
						if ( players[i].isEnabled() ) {
							if ( players[i].getText().length() > 10 )
								throw new IndexOutOfBoundsException();

							++playersAmount;
						}
					}
					
					if ( playersAmount < 2 ) {
						JOptionPane.showMessageDialog(panel,
							"Aby rozpocz gr musisz uaktywni co najmniej" +
							" dwch graczy!","Bd", JOptionPane.ERROR_MESSAGE);
					}
					else {
						// wczytaj wszystkie dane do obiektu GamePreferences
						pref = new GamePreferences(playersAmount,
								speed.getValue(),(Integer)maxScore.getValue() );

						int counter = 0;
						
						for ( int i = 0;
							  i < MAX_PLAYERS && counter < playersAmount; ++i )
						{
							if ( players[i].isEnabled() ) {
								pref.setPlayer(counter,
											   players[i].getText(),
											   colors[i].getBackground(),
											   getLeftKeyCode(i),
											   getRightKeyCode(i));
								++counter;
							}
						}
	
						// ustaw flag, e dane zostay wczytane
						done = true;
					}
				}
				catch ( IndexOutOfBoundsException ex ) {
					JOptionPane.showMessageDialog(panel,
							"Nazwa gracza moe mie maksymalnie 10 znakw",
							"Bd", JOptionPane.ERROR_MESSAGE);
				}
			}
			});
		panel.add(start);

		// przycisk "Wyjcie"
		quit = new JButton("Wyjcie");
		quit.addActionListener(new ActionListener()
			{
				@Override
				public void actionPerformed(ActionEvent e) {
				Object[] options = { "Tak, naprawd", "Nie, artowaem" };
				int n = JOptionPane.showOptionDialog(panel,
					"Chcesz ebym uwierzy, e naprawd chcesz opuci gr?",
					"Czy aby na pewno chcesz wyj?",
					JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE,
					null, options, options[1]);

				if ( n == 0 )
					System.exit(0);
				}
			});
		panel.add(quit);

//		Action action0 = new EnableAction(0);
//		Action action1 = new EnableAction(1);
//		Action action2 = new EnableAction(2);
//		Action action3 = new EnableAction(3);
//		Action action4 = new EnableAction(4);
//
//		InputMap imap = panel.getInputMap(
//								JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT);
//		imap.put(KeyStroke.getKeyStroke("1"), "player.0");
//		imap.put(KeyStroke.getKeyStroke("ctrl CONTROL"), "player.1");
//		imap.put(KeyStroke.getKeyStroke("C"), "player.2");
//		imap.put(KeyStroke.getKeyStroke("M"), "player.3");
//		imap.put(KeyStroke.getKeyStroke("LEFT"), "player.4");
//
//		ActionMap amap = panel.getActionMap();
//		amap.put("player.0", action0);
//		amap.put("player.1", action1);
//		amap.put("player.2", action2);
//		amap.put("player.3", action3);
//		amap.put("player.4", action4);

		pack();
		setResizable(false);
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		setLocationRelativeTo(null);

		start.requestFocusInWindow();
	}

	/**
	 * Zwraca <code>true</code> jeli zosta kliknity przycisk <em>Start!</em>
	 * i preferencje zostay wczytane do obiektu klasy
	 * <code>GamePreferences</code>. Mona je teraz odczyta przy uyciu metody
	 * <code>getGamePreferences()</code>.
	 * @see pl.tph.javatacka.game.GamePreferences
	 * @see pl.tph.javatacka.game.GameMenu#getGamePreferences
	 */
	public boolean isDone ( ) { return done; }
	/**
	 * Ustawia flag gotowoci menu na podan warto logiczn. T metod
	 * naley wywoa z parametrem <code>false</code> jeli chcemy uy to okno
	 * menu ponownie (po wciniciu przycisku <em>Start!</em>)
	 * @param done warto logiczna flagi gotowoci
	 */
	public void setDone ( boolean done ) { this.done = done; }

	/**
	 * Zwraca referencj na obiekt <code>GamePreferences</code>, ktry
	 * przechowuje informacje o wybranych przez graczy opcjach gry. Przed
	 * wywoaniem tej metody warto sprawdzi czy menu jest ju gotowe (czyli
	 * czy dane zostay wczytane po klikniciu przycisku <em>Start!</em>) przy
	 * pomocy metody <code>isDone()</code>.
	 * @see pl.tph.javatacka.game.GameMenu#isDone
	 */
	public GamePreferences getGamePreferences ( ) { return this.pref; }

	/**
	 * Pomocnicza metoda zwracajca kod lewego klawisza dla gracza o podanym
	 * numerze lub <code>-1</code>, gdy podany numer gracza jest niepoprawny. 
	 * @param player numer gracza (od 0 do 5)
	 * @return kod lewego klawisza
	 * @see java.awt.event.KeyEvent
	 * @see java.awt.event.MouseEvent
	 */
	private static int getLeftKeyCode ( int player ) {
		switch ( player ) {
		case 0:
			return KeyEvent.VK_1;
		case 1:
			return KeyEvent.VK_CONTROL;
		case 2:
			return KeyEvent.VK_C;
		case 3:
			return KeyEvent.VK_M;
		case 4:
			return KeyEvent.VK_LEFT;
		case 5:
			return MouseEvent.BUTTON1;
		default:
			return -1;
		}
	}
	/**
	 * Pomocnicza metoda zwracajca kod prawego klawisza dla gracza o podanym
	 * numerze lub <code>-1</code>, gdy podany numer gracza jest niepoprawny. 
	 * @param player numer gracza (od 0 do 5)
	 * @return kod prawego klawisza
	 * @see java.awt.event.KeyEvent
	 * @see java.awt.event.MouseEvent
	 */
	private static int getRightKeyCode ( int player ) {
		switch ( player ) {
		case 0:
			return KeyEvent.VK_Q;
		case 1:
			return KeyEvent.VK_WINDOWS;
		case 2:
			return KeyEvent.VK_V;
		case 3:
			return KeyEvent.VK_COMMA;
		case 4:
			return KeyEvent.VK_DOWN;
		case 5:
			return MouseEvent.BUTTON3;
		default:
			return -1;
		}
	}

	/**
	 * Maksymalna dopuszczalna (i wywietlana) liczba graczy.
	 */
	private static final int MAX_PLAYERS = 6;

	/**
	 * Flaga informujca czy zosta ju kliknity przycisk <em>Start!</em>, a
	 * ustawione opcje zostay wczytane.
	 */
	private boolean done;
	/**
	 * Obiekt przechowujcy dane o ustawionych w menu preferencjach.
	 */
	private GamePreferences pref;

	/**
	 * Gwny panel, w ktrym wywietlane s wszystkie komponenty menu.
	 */
	private JPanel panel;
	/**
	 * Przyciski wyboru kolorw dla krzywych nalecych do graczy.
	 */
	private JButton[] colors;
	/**
	 * Przyciski informujce o sterowaniu poszczeglnych graczy.
	 */
	private JButton[][] keys;
	/**
	 * Przycisk <em>Start!</em>.
	 */
	private JButton start;
	/**
	 * Przycisk <em>Wyjcie</em>.
	 */
	private JButton quit;
	/**
	 * Suwak wyboru prdkoci gry.
	 */
	private JSlider speed;
	/**
	 * Pole wyboru limitu punktw w grze.
	 */
	private JSpinner maxScore;
	/**
	 * Pola tekstowe do wpisywania nazw graczy.
	 */
	private JTextField[] players;

//	private class EnableAction extends AbstractAction {
//		private int player;
//
//		public EnableAction( int player ) {
//			this.player = player;
//		}
//
//		@Override
//		public void actionPerformed ( ActionEvent e ) {
//			if (players[player].isEnabled()) {
//				players[player].setEnabled(false);
//				colors[player].setEnabled(false);
//				keys[1][player].setEnabled(false);
//			} else {
//				players[player].setEnabled(true);
//				colors[player].setEnabled(true);
//				keys[1][player].setEnabled(true);
//			}
//		}
//	}
}

