package pl.tph.javatacka.game;

import java.awt.Color;
import pl.tph.javatacka.input.InputManager;

/**
 * Klasa <code>GamePreferences</code> suy do przechowywania informacji o
 * preferencjach dotyczcych gry. W obiektach te klasy przechowywane s dane
 * o nastpujcych opcjach gry:
 * <ul>
 * <li>ilo graczy</li>
 * <li>prdko gry</li>
 * <li>limit punktw</li>
 * <li>nazwy poszczeglnych graczy</li>
 * <li>kolory krzywych</li>
 * <li>sterowanie (pary klawiszy)</li>
 * </ul> 
 * 
 * @author Tomasz Pieczerak (tph)
 *
 */
public class GamePreferences {
	/**
	 * Tworzy nowy obiekt z preferencjami gry o podanych parametrach.
	 * @param playerAmount liczba graczy biorcych udzia w rozgrywce
	 * @param speed szybko gry
	 * @param maxScore limit punktw, po ktrym nastpi zakoczenie gry
	 */
	public GamePreferences ( int playerAmount, int speed, int maxScore ) {
		this.playerAmount = playerAmount;
		this.speed = speed;
		this.maxScore = maxScore;

		this.playerName = new String[playerAmount];
		this.playerColor = new Color[playerAmount];
		this.leftKeyCode = new int[playerAmount];
		this.rightKeyCode = new int[playerAmount];
	}

	/**
	 * Zwraca liczb graczy.
	 */
	public int getPlayerAmount ( ) { return playerAmount; }
	/**
	 * Zwraca szybko gry.
	 */
	public int getSpeed ( ) { return speed;	}
	/**
	 * Zwraca limit punktw.
	 */
	public int getMaxScore ( ) { return maxScore; }

	/**
	 * Tworzy nowego gracza o podanym numerze (indeksowane od 0) z podanymi
	 * parametrami. W przypadku, gdy numer gracz jest niepoprawny (gracz o takim
	 * numerze nie istnieje) metoda rzuci wyjtek
	 * <code>ArrayIndexOutOfBoundsException</code>.
	 * @param i numer gracza
	 * @param playerName nazwa gracza
	 * @param playerColor kolor krzywej gracza
	 * @param leftKeyCode kod lewego klawisza
	 * @param rightKeyCode kod prawego klawisza
	 * @see java.lang.ArrayIndexOutOfBoundsException
	 */
	public void setPlayer ( int i, String playerName, Color playerColor,
							int leftKeyCode, int rightKeyCode )
	{
		this.playerName[i] = playerName;
		this.playerColor[i] = playerColor;
		this.leftKeyCode[i] = leftKeyCode;
		this.rightKeyCode[i] = rightKeyCode;
	}

	/**
	 * Tworzy i zwraca referencj na obiekt klasy <code>Player</code> na
	 * podstawie przechowywanych danych. Jeli zostanie podany numer, dla
	 * ktrego odpowiadajcy gracz nie istnieje zostanie rzucony wyjtek
	 * <code>ArrayIndexOutOfBoundsException</code>.
	 * @param i numer gracza
	 * @param input menader wejcia, ktry ma obsugiwa akcje gracza 
	 * @return obiekt zawierajcy informacje o graczu
	 * @see java.lang.ArrayIndexOutOfBoundsException
	 * @see pl.tph.javatacka.game.Player
	 * @see pl.tph.javatacka.input.InputManager
	 */
	public Player getPlayer ( int i, InputManager input ) {
		Player player = new Player(playerName[i], playerColor[i],
								   leftKeyCode[i], rightKeyCode[i], input);
		return player;
	}

	/**
	 * Liczba graczy.
	 */
	private int playerAmount;
	/**
	 * Szybko gry.
	 */
	private int speed;
	/**
	 * Limit punktw, po ktrym nastpi zakoczenie gry.
	 */
	private int maxScore;

	/**
	 * Nazwy graczy.
	 */
	private String[] playerName;
	/**
	 * Kolory krzywych graczy.
	 */
	private  Color[] playerColor;

	/**
	 * Kody lewych klawiszy.
	 */
	private int[] leftKeyCode;
	/**
	 * Kody prawych klawiszy.
	 */
	private int[] rightKeyCode;
}
