package pl.tph.javatacka.graphics;

/**
 * Klasa <code>Sprite</code> jest klas bazow dla wszystkich elementw
 * graficznych niezalenie poruszajcych si po ekranie gry. Wszystkie obiekty
 * tego typu posiadaj dwa podstawowe charakteryzujce je parametry: aktualn
 * pozycj na ekranie oraz wektor prdkoci. Kady z nich zosta rozdzielony na
 * skadow pionow i poziom. Prdko jest mierzona w pikselach na
 * milisekund, dziki temu obiekty klasy <code>Sprite</code> poruszaj si z t
 * sam prdkoci <em>niezalenie od wydajnoci komputera</em>. adna ze
 * skadowych pozycji nie musi mie warto cakowitej, dziki temu moliwe jest
 * tworzenie obiektw poruszajcych si wolniej ni jeden piksel na milisekund.
 * Dodatkowo kady <code>Sprite</code> ma okrelony rozmiar (przez szeroko i
 * wysoko). Te dwa dodatkowe parametry mog by przydatne przy sprawdzaniu
 * kolizji midzy dwoma <code>Sprite'ami</code>.
 * <br><br>
 * Klasa <code>Sprite</code> jest klas abstrakcyjn, aby skorzysta z jej
 * funkcjonalnoci naley stworzy now klas pochodn do niej.
 *
 * @author Tomasz Pieczerak (tph)
 *
 */
public abstract class Sprite {
	/**
	 * Zwraca skadow poziom pozycji <code>Sprite'a</code> na ekranie.
	 * @return skadowa pozioma pooenia
	 */
	public double getX ( ) { return x; }

	/**
	 * Zwraca skadow pionow pozycji <code>Sprite'a</code> na ekranie.
	 * @return skadowa pionowa pooenia
	 */
	public double getY ( ) { return y; }

	/**
	 * Zwraca szeroko <code>Sprite'a</code>.
	 * @return szeroko (w pikselach)
	 */
	public int getWidth ( ) { return width; }

	/**
	 * Zwraca wysoko <code>Sprite'a</code>.
	 * @return wysoko (w pikselach)
	 */
	public int getHeight ( ) { return height; }

	/**
	 * Tworzy obiekt klasy <code>Sprite</code> inicjujc jego pooenie, wektor
	 * prdkoci oraz rozmiar wartociami zerowymi.
	 */
	protected Sprite ( ) {
		this.x = 0;
		this.y = 0;
		this.dx = 0;
		this.dy = 0;
		this.height = 0;
		this.width = 0;
	}

	/**
	 * Tworzy obiekt klasy <code>Sprite</code> inicjujc jego pooenie oraz
	 * wektor prdkoci podanymi wartociami.
	 *
	 * @param x skadowa pozioma pooenia obiektu na ekranie
	 * @param y skadowa pionowa pooenia obiektu na ekranie
	 * @param dx pozioma skadowa wektora prdkoci
	 * @param dy pionowa skadowa wektora prdkoci
	 * @param width
	 * @param height
	 */
	protected Sprite ( double x, double y, double dx, double dy,
					   int width, int height )
	{
		this.x = x;
		this.y = y;
		this.dx = dx;
		this.dy = dy;
		this.width = width;
		this.height = height;
	}

	/**
	 * Aktualizuje pozycj <code>Sprite'a</code> w oparciu o miniony czas.
	 * @param elapsedTime czas w milisekundach
	 */
	protected void update ( long elapsedTime ) {
		x += dx * elapsedTime;
		y += dy * elapsedTime;
	}

	/**
	 * Ustawia skadow poziom pozycji <code>Sprite'a</code> na ekranie.
	 * @param x warto skadowej poziomej pooenia
	 */
	protected void setX ( double x ) { this.x = x; }

	/**
	 * Ustawia skadow pionow pozycji <code>Sprite'a</code> na ekranie.
	 * @param y warto skadowej pionowa pooenia
	 */
	protected void setY ( double y ) { this.y = y; }

	/**
	 * Zwraca skadow poziom wektora prdkoci <code>Sprite'a</code>.
	 * @return skadowa pozioma prdkoci (podana w pikselach na milisekund)
	 */
	protected double getVelocityX ( ) { return dx; }

	/**
	 * Zwraca skadow pionow wektora prdkoci <code>Sprite'a</code>.
	 * @return skadowa pionowa prdkoci (podana w pikselach na milisekund)
	 */
	protected double getVelocityY ( ) { return dy; }

	/**
	 * Ustawia skadow poziom wektora prdkoci <code>Sprite'a</code>.
	 * @param dx warto skadowej poziomej wektora prdko (w pikselach na
	 * milisekund)
	 */
	protected void setVelocityX ( double dx ) { this.dx = dx; }

	/**
	 * Ustawia skadow pionow wektora prdkoci <code>Sprite'a</code>.
	 * @param dy warto skadowej pionowej wektora prdko (w pikselach na
	 * milisekund)
	 */
	protected void setVelocityY ( double dy ) { this.dy = dy; }

	/**
	 * Ustawia szeroko <code>Sprite'a</code>.
	 * @param width szeroko w pikselach
	 */
	protected void setWidth ( int width ) { this.width = width; }

	/**
	 * Ustawia wysoko <code>Sprite'a</code>.
	 * @param height wysoko w pikselach
	 */
	protected void setHeight ( int height ) { this.height = height; }

	/**
	 * Pozioma skadowa pooenia <code>Sprite'a</code> na ekranie.
	 */
	protected double x;
	/**
	 * Pionowa skadowa pooenia <code>Sprite'a</code> na ekranie.
	 */
	protected double y;

	/**
	 * Pozioma skadowa wektora prdkoci <code>Sprite'a</code>.
	 */
	protected double dx;
	/**
	 * Pionowa skadowa wektora prdkoci <code>Sprite'a</code>.
	 */
	protected double dy;

	/**
	 * Szeroko <code>Sprite'a</code> (w pikselach).
	 */
	protected int width;
	/**
	 * Wysoko <code>Sprite'a</code> (w pikselach).
	 */
	protected int height;
}
